import asyncio
import json
import os
import time
from datetime import datetime, timedelta

import boto3
import pytz

from src import daily_report_generator, monthly_report_generator
from src.constants import DAILY_JOB_TYPE, MONTHLY_JOB_TYPE
from src.daily_report_generator import get_daily_report_key
from src.monthly_report_generator import get_monthly_report_key
from src.services.recipients_service import RecipientsException, get_recipients
from src.services.notification_email_service import send_email_sqs, EmailSentError, prepare_email_notification_payload
from src.util.logger import logger
from src.util.metrics_client import MetricsClient
from src.util.utils import put_to_s3, from_s3, generate_pdf_from_html, HTMLToPDFConversionError

AWS_REGION = os.getenv("AWS_REGION", "us-east-1")
ORG_DAILY_BUCKET = os.getenv("ORG_DAILY_BUCKET", "kcdev01-mdr-reporting-job-infra-daily-alert")
ORG_MONTHLY_BUCKET = os.getenv("ORG_MONTHLY_BUCKET", "kcdev01-mdr-reporting-job-infra-monthly-alert")
REPORT_PDF_BUCKET = os.getenv("REPORT_PDF_BUCKET", "kcdev01-mdr-reporting-generator-infra-report-pdf")
TIME_ZONE = os.getenv("TIME_ZONE", "America/New_York")

DEVELOPMENT_RECIPIENTS = ['daniel.chuang@broadcom.com', 'bruce.deakyne@broadcom.com',
                          'meghna.patil@broadcom.com', 'vaishali.kale@broadcom.com',
                          'jatin.patel@broadcom.com', 'patejo@vmware.com']

def get_alert_data_key(message_body):
    # Parse the message body as JSON
    message = json.loads(message_body)

    # The structure of the message is such that the object key
    # is located within 'Records', then within 's3' and 'object'
    if 'Records' in message and len(message['Records']) > 0:
        s3_event = message['Records'][0]
        if 's3' in s3_event and 'object' in s3_event['s3']:
            bucket_name = s3_event['s3']['bucket']['name']
            object_key = s3_event['s3']['object']['key']
            return [bucket_name, object_key]
            # Add your processing logic here
        else:
            print("Could not find 's3' or 'object' in the message.")
            # TODO: raise exception
            return None
    else:
        print("No 'Records' found in the message.")
        return None


async def consume_message(sqs, queue_url, notification_queue_url, metrics_client):
    # 1. Receive single message from SQS queue when daily/monthly cronjob creates raw statistic data in S3
    # bucket. Long polling for s3 object creation event
    response = sqs.receive_message(
        QueueUrl=queue_url,
        MaxNumberOfMessages=1,
        WaitTimeSeconds=20,
        VisibilityTimeout=60
    )
    if 'Messages' not in response:
        logger.debug(f"No message from SQS.")
        return
    message = response['Messages'][0]
    receipt_handle = message['ReceiptHandle']
    bucket_name, alert_obj_key = get_alert_data_key(message['Body'])
    job_type = DAILY_JOB_TYPE if 'daily' in bucket_name else MONTHLY_JOB_TYPE
    raw_data = from_s3(alert_obj_key, bucket_name)
    org_key = raw_data.get("org_key")
    # 2. Process the message and generate report PDF
    try:
        logger.info(f"Processing orgKey: [{org_key}] object: [{alert_obj_key}] in bucket [{bucket_name}]")
        report_s3_url = None
        report_recipients = get_recipients(org_key)
        logger.info(f'Fetched recipients: {report_recipients}')
        subscribed_recipients = []
        report_process_start = time.time()
        if job_type is DAILY_JOB_TYPE:
            timezone = pytz.timezone(TIME_ZONE)
            report_date = datetime.now(timezone) - timedelta(days=1)
            logger.info(
                f'[{org_key}] Start processing daily statistic data for {report_date.strftime("%Y-%m-%d")}')
            html_file = daily_report_generator.daily_report_generator(raw_data, report_date)
            logger.info(f"[{org_key}] Processed raw data and creating daily report html")
            pdf_file = f'{org_key}_daily_report.pdf'
            await generate_pdf_from_html(html_file, pdf_file)
            logger.info(f"[{org_key}] Converted daily report html to pdf.")
            report_s3_url = put_to_s3(filepath=pdf_file,
                                      s3_key=get_daily_report_key(org_key, report_date.strftime("%Y-%m-%d")),
                                      bucket=REPORT_PDF_BUCKET)
            logger.info(f"[{org_key}] Pushed daily report pdf. Proceed to delete SQS message.")
            metrics_client.request_latency.labels(
                "generator", DAILY_JOB_TYPE, org_key
            ).observe(time.time() - report_process_start)
            metrics_client.org_count.labels(
                "generator", DAILY_JOB_TYPE, org_key
            ).inc()
            for recipient in report_recipients:
                if 'daily_summary' in recipient and recipient['daily_summary']:
                    subscribed_recipients.append(recipient['email_address'])
        if job_type is MONTHLY_JOB_TYPE:
            logger.info(f'[{org_key}] Start processing monthly statistic data.')
            html_file = monthly_report_generator.monthly_report_generator(raw_data)
            pdf_file = f'{org_key}_monthly_report.pdf'
            logger.info(f"[{org_key}] Processed raw data and creating monthly report html")
            await generate_pdf_from_html(html_file, pdf_file)
            logger.info(f"[{org_key}] Converted monthly report html to pdf.")
            report_s3_url = put_to_s3(filepath=pdf_file,
                                      s3_key=get_monthly_report_key(org_key),
                                      bucket=REPORT_PDF_BUCKET)
            logger.info(f"[{org_key}] Pushed monthly report pdf. Proceed to delete SQS message.")
            metrics_client.request_latency.labels(
                "generator", MONTHLY_JOB_TYPE, org_key
            ).observe(time.time() - report_process_start)
            metrics_client.org_count.labels(
                "generator", MONTHLY_JOB_TYPE, org_key
            ).inc()
            for recipient in report_recipients:
                if 'monthly_reports' in recipient and recipient['monthly_reports']:
                    subscribed_recipients.append(recipient['email_address'])

        is_production = os.getenv('ENV_CATEGORY', 'int') == 'prd'
        # Avoid sending emails to external addresses in non production envs.
        if not is_production:
            subscribed_recipients = DEVELOPMENT_RECIPIENTS
            logger.info(f'DEV/STAGING: Sending to internal recipients: {subscribed_recipients}')
        else:
            logger.info(f'PROD: Sending email to recipients: [{subscribed_recipients}]')

        # 3. Send email request to notification sender SQS
        if len(subscribed_recipients) > 0:
            logger.info(f'[{org_key}] [{job_type}] subscribed recipients: [{subscribed_recipients}].')
            email_payload = prepare_email_notification_payload(report_s3_url, org_key, subscribed_recipients, job_type)
            if notification_queue_url:
                send_email_sqs(sqs, notification_queue_url, email_payload)
                logger.info(f'[{org_key}] [{job_type}] Sent email request to notification-sender SQS')
            else:
                # TODO: CBC-36825 Enable notification sender SQS once it's deployed to prod
                logger.info(f'[{org_key}] [{job_type}] SQS url is not set [{notification_queue_url}]. '
                            f'SQS payload: {email_payload} Skip sending email')
        else:
            logger.info(f'[{org_key}] [{job_type}] No email recipient configured. Skip sending email')

        # 4. Delete received message from queue if it's processed successfully.
        logger.info(f'[{org_key}] [{job_type}] Completed generating and sending report to SQS, delete message.')
        sqs.delete_message(
            QueueUrl=queue_url,
            ReceiptHandle=receipt_handle
        )
    except HTMLToPDFConversionError as conversionError:
        logger.exception(f'[{org_key}] [{job_type}] Failed to convert report {conversionError}')
        metrics_client.org_err.labels(
            "generator", job_type, org_key, "html_conversion"
        ).inc()
    except EmailSentError as emailSentError:
        logger.exception(f'[{org_key}] [{job_type}] Failed to send email request {emailSentError}')
        metrics_client.org_err.labels(
            "generator", job_type, org_key, "email_sent"
        ).inc()
    except RecipientsException as receiptError:
        logger.exception(f'[{org_key}] [{job_type}] Failed to fetch recipient from MDR report web server {receiptError}')
        metrics_client.org_err.labels(
            "generator", job_type, org_key, "fetch_recipient"
        ).inc()
    except Exception as e:
        logger.exception(f"[{org_key}] [{job_type}] Exception while generating reports: {e}")
        metrics_client.org_err.labels(
            "generator", job_type, org_key, "unknown"
        ).inc()


async def main():
    metrics_client = MetricsClient(
        metric_prefix="mdr_reporting_prometheus",
        metrics_port=9090,
    )

    # Initialize the SQS client
    sqs = boto3.client('sqs', region_name=AWS_REGION)

    # Specify your queue URL here
    queue_url = os.getenv("ALERT_BUCKET_EVENT_SQS_URL", None)
    notification_queue_url = os.getenv("NOTIFICATION_SENDER_SQS_URL", None)

    while True:
        await consume_message(sqs, queue_url, notification_queue_url, metrics_client)


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())
