import os

from marshmallow import ValidationError
from typing import List

from datetime import datetime, timedelta

from src.constants import DAILY_JOB_TYPE, MONTHLY_JOB_TYPE
from src.models.email import NotificationType, EmailSchema
from src.services.email_content_render import render_templates
from src.services.recipients_service import get_monthly_report_template
from src.util.logger import logger

FROM_EMAIL = os.getenv("FROM_EMAIL", "noreply@cbdtest.io")


class EmailSentError(Exception):
    def __init__(self, message):
        super().__init__(message)


class InvalidReportTypeError(Exception):
    def __init__(self, message):
        super().__init__(message)


def prepare_email_notification_payload(report_s3_url: str, org_key: str, recipients: List[str], report_type):
    """
    TODO: Implements 'report_present'
    :param report_type:
    :param recipients:
    :param org_key:
    :param report_s3_url:
    :return:
    """
    is_production = os.getenv('ENV_CATEGORY', 'int') == 'prd'
    try:
        if report_type == DAILY_JOB_TYPE:
            report_date = (datetime.today() - timedelta(days=1))
            subj, html_body, text_body = render_templates(template_type='dailyReport',
                                                          context={'date': report_date.strftime("%Y-%m-%d"),
                                                                   'no_alerts': False,
                                                                   'sku': 'mdr'})
        elif report_type == MONTHLY_JOB_TYPE:
            subj, html_body, text_body = render_templates(template_type='monthlyReport',
                                                          context={'comments': get_monthly_report_template(),
                                                                   'is_production': is_production})
        else:
            raise InvalidReportTypeError(f"Unknown report type {report_type}")

        email = {
            'notificationType': NotificationType.REPORTING.value,
            'senderEmail': FROM_EMAIL,
            'recipientEmailList': recipients,
            'bccEmails': ["cb.mdr@broadcom.com"],
            'subject': subj,
            'htmlContent': html_body,
            'textContent': text_body,
            's3URI': report_s3_url,
            'reportPresent': True,
            'orgKey': org_key
        }
        email_schema = EmailSchema()
        res = email_schema.load(email)
        return EmailSchema().dumps(res)
    except ValidationError as err:
        logger.exception(f'Failed to generate email payload {err.messages}')
        raise err


def send_email_sqs(sqs_client, notification_queue_url: str, sqs_payload: str):
    try:
        logger.debug(f'Sending SQS payload {sqs_payload}')
        response = sqs_client.send_message(
            QueueUrl=notification_queue_url,
            MessageBody=sqs_payload
        )
        if 'MessageId' in response:
            return response['MessageId']
        return ''
    except Exception as err:
        raise EmailSentError(f'Failed to send payload to SQS {notification_queue_url}. '
                             f'Payload: {sqs_payload}. Exception {err}')
