import os

import requests
from marshmallow import Schema, fields, EXCLUDE

from src.util.logger import logger

host = os.getenv('INTERNAL_INGRESS_URL', "test-host")
api_key = os.getenv('API_KEY')
connector_id = os.getenv('CONNECTOR_ID')


class RecipientsException(Exception):
    def __init__(self, message):
        super().__init__(message)


class RecipientSchema(Schema):
    id = fields.Str()
    org_key = fields.Str()
    email_address = fields.Email(required=True)
    daily_summary = fields.Boolean()
    monthly_reports = fields.Boolean()

    class Meta:
        unknown = EXCLUDE


class ResultSchema(Schema):
    results = fields.List(fields.Nested(RecipientSchema()))

    class Meta:
        unknown = EXCLUDE


def get_recipients(org_key: str) -> list:
    url = f'https://{host}/mdr/reporting/v1/orgs/{org_key}/recipients/_search'
    # search all recipients
    request_body = {
        "query": ""
    }
    headers = {
        "Content-type": "application/json",
        "X-Auth-Token": f'{api_key}/{connector_id}',
    }
    try:
        logger.debug(f'[{org_key}] Send search recipient request to url={url}')
        response = requests.post(url, json=request_body, headers=headers)

        if response.status_code == 200:
            data = response.json()
            logger.debug(f'[{org_key}] Got recipients: {data}')
            result_schema = ResultSchema()
            result = result_schema.load(data)
            return result['results']
        else:
            logger.error(f'[{org_key}] Failed to get email recipients. Status code: [{response.status_code}]')
            raise RecipientsException
    except Exception as ex:
        logger.exception(f'[{org_key}] Failed to get email recipients. Exception: {ex}')
        raise RecipientsException(ex)


def get_monthly_report_template() -> str:
    url = f'https://{host}/mdr/reporting/v1/private/monthly_email_template'
    headers = {
        "Content-type": "application/json",
        "X-Auth-Token": f'{api_key}/{connector_id}',
    }
    try:
        logger.info(f'Get monthly report email template. url={url}')
        response = requests.get(url, headers=headers)

        if response.status_code == 200:
            data = response.json()
            return data['email_body']
        else:
            raise RecipientsException
    except Exception as ex:
        raise RecipientsException(ex)
