# color for each severity
import numpy as np
from matplotlib import pyplot as plt

from src.constants import ALERT_SOURCE_TYPE_DEV

SEVERITY_5 = '#E7D5EE'
SEVERITY_6 = 'plum'
SEVERITY_7 = 'mediumorchid'
SEVERITY_8 = '#D3E7FF'
SEVERITY_9 = '#68ACFF'
SEVERITY_10 = '#2475D9'

LABEL_MAX_LENGTH = 15
MAX_BARS = 6


def plot_pie_severity(pie_severity_scores, alert_type, report_type, org_key):
    """
    Plot pie chart of alert severity distributions today.
    :param org_key:
    :param pie_severity_scores: processed severity scores data
    :param alert_type: 'likely' or 'mdr' alert
    :param report_type: 'daily' or 'monthly'
    :return:
    """
    if not pie_severity_scores or all(x == 0 for x in list(pie_severity_scores.values())):
        return 'src/default_image.png'
    fig = plt.figure(figsize=(80, 25))
    ax = plt.gca()
    severity_counts = []
    my_labels = []

    slice_colors = []  # This will be the list of colors for each slice
    colors = {
        '5': SEVERITY_5, '6': SEVERITY_6, '7': SEVERITY_7, '8': SEVERITY_8, '9': SEVERITY_9, '10': SEVERITY_10
    }
    for k in list(pie_severity_scores.keys()):
        if pie_severity_scores[k] != 0:
            severity_counts.append(pie_severity_scores[k])
            slice_colors.append(colors[k])
            if int(k) >= 8:
                my_labels.append(str(k) + ' (Priority)')
            else:
                my_labels.append(str(k) + ' (Eligible)')
    legend_labels = [f'{label}: {size}' for label, size in zip(my_labels, severity_counts)]

    x = np.array(severity_counts)
    wedges, texts = ax.pie(x, radius=1.1, colors=slice_colors,
                           textprops={'weight': 'heavy', 'fontsize': 50})
    plt.legend(wedges, legend_labels, loc="center left", bbox_to_anchor=(1, 0, 0.5, 1),
               fontsize=48, title_fontsize=56)
    ax.set(xlabel=None)
    ax.set(ylabel=None)
    plot_name = f'{org_key}_{report_type}_pie_severity_{alert_type}.svg'
    fig.savefig(plot_name, format='svg', bbox_inches='tight')
    plt.close()
    return plot_name


def plot_horizontal_bars(names, counts, alert_type, alert_source_type, report_type, org_key):
    fig = plt.figure(figsize=(32, 19))
    if not names:
        return 'src/default_image.png'
    ax = plt.gca()
    bar_color = 'mediumorchid' if alert_source_type == ALERT_SOURCE_TYPE_DEV else '#2475D9'
    for name_idx in range(len(names)):
        if len(names[name_idx]) > LABEL_MAX_LENGTH:
            name = names[name_idx]
            names[name_idx] = '\n'.join([name[i:i + LABEL_MAX_LENGTH] for i in range(0, len(name), LABEL_MAX_LENGTH)])
    n_bars = len(counts)
    if n_bars == 1:
        positions = [MAX_BARS / 2]  # Center the single bar
    else:
        start_pos = (MAX_BARS - n_bars) / 2
        positions = [start_pos + i for i in range(n_bars)]
    bars = ax.barh(positions, counts, color=bar_color, height=0.4)
    ax.set_yticks(positions)
    ax.set_yticklabels(names)
    plt.ylim(-0.5, 5.5)
    for bar, value in zip(bars, counts):
        text_x_position = bar.get_width() + 0.1
        ax.text(text_x_position, bar.get_y() + bar.get_height() / 2, str(value),
                va='center', ha='left', fontsize=36)
    ax.tick_params(axis='x', which='major', labelsize=36, bottom=False)
    ax.tick_params(axis='y', which='major', labelsize=36, left=False)
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.set(xlabel=None)
    ax.set(ylabel=None)
    fig.subplots_adjust(left=0.25)
    plot_name = f'{org_key}_{report_type}_barh_{alert_type}_{alert_source_type}.svg'
    fig.savefig(plot_name, format='svg', bbox_inches='tight')
    return plot_name
